@@ -23,9 +23,11 @@
 
 # pylint: disable=unused-import
 
-from lava_dispatcher.pipeline.actions.boot.qemu import BootQEMU, VirtualMachine
+from lava_dispatcher.pipeline.actions.boot.qemu import BootQEMU
 from lava_dispatcher.pipeline.actions.boot.u_boot import UBoot
 from lava_dispatcher.pipeline.actions.boot.kexec import BootKExec
 from lava_dispatcher.pipeline.actions.boot.ssh import SshLogin, Schroot
 from lava_dispatcher.pipeline.actions.boot.fastboot import BootFastboot
 from lava_dispatcher.pipeline.actions.boot.uefi_menu import UefiMenu
+from lava_dispatcher.pipeline.actions.boot.lxc import BootLxc
+from lava_dispatcher.pipeline.actions.boot.ipxe import IPXE
